# -*- coding:utf-8 -*-
#---------------------------------------------------------------------
# 
# Visor de PyQGIS
#
# Copyright (C) 2010  Germán Carrillo
#---------------------------------------------------------------------
# 
# licensed under the terms of GNU GPL 2
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software. If not, see <http://www.gnu.org/licenses/>.
# 
#---------------------------------------------------------------------

from os.path import isfile

from PyQt4.QtCore import Qt, QObject, SIGNAL, QVariant
from PyQt4.QtGui import ( QDialog, QFileDialog, QListWidgetItem, QIcon, 
                          QDialogButtonBox, QMessageBox )

from dlgCargueSpatiaLite_ui import Ui_CargueSpatiaLite

class DialogoCargueSpatiaLite( QDialog, Ui_CargueSpatiaLite ):
    """ Open a dialog to load SpatiaLite layers """
    def __init__( self, parent ):
        self.parent = parent
        QDialog.__init__( self, self.parent )
        self.setupUi( self )

        self.dbLayers = {} # layer name:[layer name, geometry column, geometry type, srid]
        self.layers = {} # layer name: geometry column
        self.dbPath = ''

        self.btnAceptar = self.buttonBox.button( QDialogButtonBox.Ok )
        self.btnAceptar.setEnabled( False )

        # SIGNALS - SLOTS
        self.connect( self.lstCapas, SIGNAL( "itemSelectionChanged()" ), self.selectionChanged )
        self.connect( self.btnAbrir, SIGNAL( "clicked()" ), self.openFileDialog )
        QObject.connect( self.buttonBox, SIGNAL( "accepted()" ), self.aceptar )

    def openFileDialog( self ):
        """ Open a dialog to locate the sqlite file """        
        path = QFileDialog.getOpenFileName( self.parent, "Abrir base de datos de SpatiaLite",
                '.', "SpatiaLite (*.sqlite)" )

        if path: self.dbPath = path # To make possible cancel the FileDialog and continue loading a predefined db

        self.openDBFile( self.dbPath )

    def openDBFile( self, path ):
        """ Open the SpatiaLite file to extract info about geographic tables """
        if isfile( unicode( path ) ):
            self.txtRuta.setText( path )
            self.lstCapas.clear()

            # Execute a query in SQLite to show the available layers
            from sqlite3 import connect
            conn = connect( str( self.dbPath ) )
            cursor = conn.cursor()
            cursor.execute( "SELECT f_table_name, f_geometry_column, type, srid FROM 'geometry_columns'" )

            self.dbLayers = {} 
            for row in cursor:
                self.dbLayers[ row[ 0 ] ] = row # Load the layer info into the dictionary

            if len( self.dbLayers ):
                self.cargarCapasALista()
            else:
                msg = QMessageBox(QMessageBox.Warning, 'Advertencia',
                    'La base de datos de SpatiaLite no contiene \n' \
                    + 'tablas con datos geográficos.', QMessageBox.Ok, self)
                msg.addButton('Aceptar', QMessageBox.AcceptRole)
                msg.exec_()        

    def cargarCapasALista( self ):
        """ Load available layers into the list """
        for layerName in self.dbLayers:
            # Geometry handling
            geom = self.dbLayers[ layerName ][ 2 ]
            if geom == "POINT" or geom == 'MULTIPOINT':
                icono= QIcon( ":/imgs/imagenes/punto.png" )
            elif geom == "LINESTRING" or geom == "MULTILINESTRING" or geom == "LINESTRINGM":
                icono= QIcon( ":/imgs/imagenes/linea.png" )
            elif geom == "POLYGON" or geom == "MULTIPOLYGON":
                icono= QIcon( ":/imgs/imagenes/poligono.png" )
            else:
                raise RuntimeError, "Geometría desconocida: " + str( geom )

            newItem = QListWidgetItem( icono, layerName )
            newItem.setData( Qt.UserRole, QVariant( self.dbLayers[ layerName ][ 1 ] ) )
            self.lstCapas.addItem( newItem )            

    def selectionChanged( self ):
        """ Enable/disable the Ok button """
        if len( self.lstCapas.selectedItems() ) > 0:
            self.btnAceptar.setEnabled( True )
        else:
            self.btnAceptar.setEnabled( False )

    def mostrar( self ):
        """ Show the dialog """
        return self.exec_()

    def aceptar( self ):
        """ Load the selected layers into the dictionary """
        for item in self.lstCapas.selectedItems():
            self.layers[ item.text() ] = item.data( Qt.UserRole ).toString()

